@@ -4,13 +4,15 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Initialization of the Dash integration."""
+
 from zenml.integrations.constants import DASH
 from zenml.integrations.integration import Integration
 
@@ -23,6 +25,7 @@ class DashIntegration(Integration):
         "dash>=2.0.0",
         "dash-cytoscape>=0.3.0",
         "dash-bootstrap-components>=1.0.1",
+        "jupyter-dash>=0.4.2",
     ]
 
 
