@@ -4,15 +4,15 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Implementation of the pipeline run lineage visualizer."""
 
-from abc import abstractmethod
 from typing import Any, Collection, Dict, List, Union
 
 import dash
@@ -21,12 +21,12 @@ import dash_cytoscape as cyto
 from dash import dcc, html
 from dash.dependencies import Input, Output
 
+from zenml.cli import utils as cli_utils
 from zenml.enums import ExecutionStatus
+from zenml.environment import Environment
 from zenml.logger import get_logger
-from zenml.post_execution.pipeline_run import PipelineRunView
-from zenml.visualizers.base_pipeline_run_visualizer import (
-    BasePipelineRunVisualizer,
-)
+from zenml.post_execution import PipelineRunView
+from zenml.visualizers import BasePipelineRunVisualizer
 
 logger = get_logger(__name__)
 
@@ -98,9 +98,7 @@ STYLESHEET = [
 
 
 class PipelineRunLineageVisualizer(BasePipelineRunVisualizer):
-    """Implementation of a lineage diagram via the [dash](
-    https://plotly.com/dash/) and [dash-cyctoscape](
-    https://dash.plotly.com/cytoscape) library."""
+    """Implementation of a lineage diagram via the dash and dash-cytoscape libraries."""
 
     ARTIFACT_PREFIX = "artifact_"
     STEP_PREFIX = "step_"
@@ -111,21 +109,52 @@ class PipelineRunLineageVisualizer(BasePipelineRunVisualizer):
         ExecutionStatus.COMPLETED: "blue",
     }
 
-    @abstractmethod
     def visualize(
-        self, object: PipelineRunView, *args: Any, **kwargs: Any
+        self,
+        object: PipelineRunView,
+        magic: bool = False,
+        *args: Any,
+        **kwargs: Any,
     ) -> dash.Dash:
-        """Method to visualize pipeline runs via the Dash library. The layout
-        puts every layer of the dag in a column.
-        """
+        """Method to visualize pipeline runs via the Dash library.
 
-        app = dash.Dash(
-            __name__,
-            external_stylesheets=[
-                dbc.themes.BOOTSTRAP,
-                dbc.icons.BOOTSTRAP,
-            ],
-        )
+        The layout puts every layer of the dag in a column.
+
+        Args:
+            object: The pipeline run to visualize.
+            magic: If True, the visualization is rendered in a magic mode.
+            *args: Additional positional arguments.
+            **kwargs: Additional keyword arguments.
+
+        Returns:
+            The Dash application.
+        """
+        external_stylesheets = [
+            dbc.themes.BOOTSTRAP,
+            dbc.icons.BOOTSTRAP,
+        ]
+        if magic:
+            if Environment.in_notebook:
+                # Only import jupyter_dash in this case
+                from jupyter_dash import JupyterDash  # noqa
+
+                JupyterDash.infer_jupyter_proxy_config()
+
+                app = JupyterDash(
+                    __name__,
+                    external_stylesheets=external_stylesheets,
+                )
+                mode = "inline"
+            else:
+                cli_utils.warning(
+                    "Cannot set magic flag in non-notebook environments."
+                )
+        else:
+            app = dash.Dash(
+                __name__,
+                external_stylesheets=external_stylesheets,
+            )
+            mode = None
         nodes, edges, first_step_id = [], [], None
         first_step_id = None
         for step in object.steps:
@@ -143,7 +172,8 @@ class PipelineRunLineageVisualizer(BasePipelineRunVisualizer):
                     "data": {
                         "id": step_id,
                         "execution_id": execution_id,
-                        "label": f"{execution_id} / {step.name}",
+                        "label": f"{execution_id} / {step.entrypoint_name}",
+                        "entrypoint_name": step.entrypoint_name,  # redundant for consistency
                         "name": step.name,  # redundant for consistency
                         "type": "step",
                         "parameters": step.parameters,
@@ -302,7 +332,14 @@ class PipelineRunLineageVisualizer(BasePipelineRunVisualizer):
             Input("cytoscape", "selectedNodeData"),
         )
         def display_data(data_list: List[Dict[str, Any]]) -> str:
-            """Callback for the text area below the graph"""
+            """Callback for the text area below the graph.
+
+            Args:
+                data_list: The selected node data.
+
+            Returns:
+                str: The selected node data.
+            """
             if data_list is None:
                 return "Click on a node in the diagram."
 
@@ -337,9 +374,18 @@ class PipelineRunLineageVisualizer(BasePipelineRunVisualizer):
         def reset_layout(
             n_clicks: int,
         ) -> List[Union[int, List[Dict[str, Collection[str]]]]]:
-            """Resets the layout"""
+            """Resets the layout.
+
+            Args:
+                n_clicks: The number of clicks on the reset button.
+
+            Returns:
+                The zoom and the elements.
+            """
             logger.debug(n_clicks, "clicked in reset button.")
             return [1, edges + nodes]
 
+        if mode is not None:
+            app.run_server(mode=mode)
         app.run_server()
         return app
