@@ -4,22 +4,25 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Initialization of the Graphviz integration."""
+
 from zenml.integrations.constants import GRAPHVIZ
 from zenml.integrations.integration import Integration
 
 
 class GraphvizIntegration(Integration):
-    """Definition of Plotly integration for ZenML."""
+    """Definition of Graphviz integration for ZenML."""
 
     NAME = GRAPHVIZ
     REQUIREMENTS = ["graphviz>=0.17"]
+    SYSTEM_REQUIREMENTS = {"graphviz": "dot"}
 
 
 GraphvizIntegration.check_installation()
