@@ -4,13 +4,14 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Implementation of the Graphviz pipeline run DAG visualizer."""
 
 import tempfile
 from abc import abstractmethod
@@ -19,10 +20,8 @@ from typing import Any
 import graphviz
 
 from zenml.logger import get_logger
-from zenml.post_execution.pipeline_run import PipelineRunView
-from zenml.visualizers.base_pipeline_run_visualizer import (
-    BasePipelineRunVisualizer,
-)
+from zenml.post_execution import PipelineRunView
+from zenml.visualizers import BasePipelineRunVisualizer
 
 logger = get_logger(__name__)
 
@@ -46,7 +45,16 @@ class PipelineRunDagVisualizer(BasePipelineRunVisualizer):
     def visualize(
         self, object: PipelineRunView, *args: Any, **kwargs: Any
     ) -> graphviz.Digraph:
-        """Creates a pipeline lineage diagram using graphviz."""
+        """Creates a pipeline lineage diagram using graphviz.
+
+        Args:
+            object: The pipeline run view to visualize.
+            *args: Additional arguments to pass to the visualization.
+            **kwargs: Additional keyword arguments to pass to the visualization.
+
+        Returns:
+            A graphviz digraph object.
+        """
         logger.warning(
             "This integration is not completed yet. Results might be unexpected."
         )
@@ -58,7 +66,7 @@ class PipelineRunDagVisualizer(BasePipelineRunVisualizer):
             # add each step as a node
             dot.node(
                 self.STEP_PREFIX + str(step.id),
-                step.name,
+                step.entrypoint_name,
                 shape=self.STEP_SHAPE,
             )
             # for each parent of a step, add an edge
