@@ -4,13 +4,14 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Implementation of the Plotly Pipeline Lineage Visualizer."""
 
 from abc import abstractmethod
 from typing import Any
@@ -20,8 +21,8 @@ import plotly.express as px
 from plotly.graph_objs import Figure
 
 from zenml.logger import get_logger
-from zenml.post_execution.pipeline import PipelineView
-from zenml.visualizers.base_pipeline_visualizer import BasePipelineVisualizer
+from zenml.post_execution import PipelineView
+from zenml.visualizers import BasePipelineVisualizer
 
 logger = get_logger(__name__)
 
@@ -36,7 +37,16 @@ class PipelineLineageVisualizer(BasePipelineVisualizer):
     def visualize(
         self, object: PipelineView, *args: Any, **kwargs: Any
     ) -> Figure:
-        """Creates a pipeline lineage diagram using plotly."""
+        """Creates a pipeline lineage diagram using plotly.
+
+        Args:
+            object: The pipeline view to visualize.
+            *args: Additional arguments to pass to the visualization.
+            **kwargs: Additional keyword arguments to pass to the visualization.
+
+        Returns:
+            A plotly figure.
+        """
         logger.warning(
             "This integration is not completed yet. Results might be unexpected."
         )
@@ -48,11 +58,11 @@ class PipelineLineageVisualizer(BasePipelineVisualizer):
             for step in run.steps:
                 category_dict[run.name].update(
                     {
-                        step.name: str(step.id),
+                        step.entrypoint_name: str(step.id),
                     }
                 )
-                if step.name not in dimensions:
-                    dimensions.append(f"{step.name}")
+                if step.entrypoint_name not in dimensions:
+                    dimensions.append(f"{step.entrypoint_name}")
 
         category_df = pd.DataFrame.from_dict(category_dict, orient="index")
 
