@@ -4,13 +4,14 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Initialization of the PyTorch integration."""
 
 from zenml.integrations.constants import PYTORCH
 from zenml.integrations.integration import Integration
@@ -26,9 +27,7 @@ class PytorchIntegration(Integration):
     @classmethod
     def activate(cls) -> None:
         """Activates the integration."""
-        import_class_by_path(
-            "zenml.integrations.pytorch.materializers.pytorch_materializer.PyTorchMaterializer"
-        )
+        from zenml.integrations.pytorch import materializers  # noqa
 
 
 PytorchIntegration.check_installation()
