@@ -4,15 +4,15 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
-"""
-The `io` module handles file operations for the ZenML package. It offers a
-standard interface for reading, writing and manipulating files and directories.
-It is heavily influenced and inspired by the `io` module of `tfx`.
+"""The `io` module handles file operations for the ZenML package.
+
+It offers a standard interface for reading, writing and manipulating files and
+directories. It is heavily influenced and inspired by the `io` module of `tfx`.
 """
