@@ -1,27 +1,32 @@
-#  Copyright (c) ZenML GmbH 2021. All Rights Reserved.
+#  Copyright (c) ZenML GmbH 2022. All Rights Reserved.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
-"""
+"""Initialization of ZenML materializers.
+
 Materializers are used to convert a ZenML artifact into a specific format. They
 are most often used to handle the input or output of ZenML steps, and can be
 extended by building on the `BaseMaterializer` class.
 """
 
-from zenml.logger import get_logger
-from zenml.materializers.built_in_materializer import (  # noqa
-    BuiltInMaterializer,
-)
-from zenml.materializers.numpy_materializer import NumpyMaterializer  # noqa
-from zenml.materializers.pandas_materializer import PandasMaterializer  # noqa
 
-logger = get_logger(__name__)
+from zenml.materializers.built_in_materializer import BuiltInMaterializer
+from zenml.materializers.numpy_materializer import NumpyMaterializer
+from zenml.materializers.pandas_materializer import PandasMaterializer
+from zenml.materializers.service_materializer import ServiceMaterializer
+
+__all__ = [
+    "BuiltInMaterializer",
+    "NumpyMaterializer",
+    "PandasMaterializer",
+    "ServiceMaterializer",
+]
