@@ -4,16 +4,16 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
-"""
-A ZenML pipeline is a sequence of tasks that execute in a specific order and
-yield artifacts. The artifacts are stored within the artifact store and indexed
+"""A ZenML pipeline consists of tasks that execute in order and yield artifacts.
+
+The artifacts are stored within the artifact store and indexed
 via the metadata store. Each individual task within a pipeline is known as a
 step. The standard pipelines within ZenML are designed to have easy interfaces
 to add pre-decided steps, with the order also pre-decided. Other sorts of
@@ -23,5 +23,8 @@ Pipelines can be written as simple functions. They are created by using decorato
 """
 
 
-from zenml.pipelines.base_pipeline import BasePipeline  # noqa
-from zenml.pipelines.pipeline_decorator import pipeline  # noqa
+from zenml.pipelines.base_pipeline import BasePipeline
+from zenml.pipelines.pipeline_decorator import pipeline
+from zenml.pipelines.schedule import Schedule
+
+__all__ = ["BasePipeline", "pipeline", "Schedule"]
