@@ -4,14 +4,15 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
-"""
+"""Initialization for the post-execution module.
+
 After executing a pipeline, the user needs to be able to fetch it from history
 and perform certain tasks. The post_execution submodule provides a set of
 interfaces with which the user can interact with artifacts, the pipeline, steps,
@@ -23,9 +24,4 @@ from zenml.post_execution.pipeline import PipelineView
 from zenml.post_execution.pipeline_run import PipelineRunView
 from zenml.post_execution.step import StepView
 
-__all__ = [
-    "PipelineView",
-    "PipelineRunView",
-    "StepView",
-    "ArtifactView",
-]
+__all__ = ["PipelineView", "PipelineRunView", "StepView", "ArtifactView"]
