@@ -4,13 +4,14 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
+"""Initialization for the post-execution artifact class."""
 
 from typing import TYPE_CHECKING, Any, Optional, Type
 
@@ -19,15 +20,17 @@ from zenml.utils import source_utils
 
 if TYPE_CHECKING:
     from zenml.materializers.base_materializer import BaseMaterializer
-    from zenml.metadata.base_metadata_store import BaseMetadataStore
+    from zenml.metadata_stores import BaseMetadataStore
     from zenml.post_execution.step import StepView
 
 logger = get_logger(__name__)
 
 
 class ArtifactView:
-    """Post-execution artifact class which can be used to read
-    artifact data that was created during a pipeline execution.
+    """Post-execution artifact class.
+
+    This can be used to read artifact data that was created during a pipeline
+    execution.
     """
 
     def __init__(
@@ -68,40 +71,69 @@ class ArtifactView:
 
     @property
     def id(self) -> int:
-        """Returns the artifact id."""
+        """Returns the artifact id.
+
+        Returns:
+            The artifact id.
+        """
         return self._id
 
     @property
     def type(self) -> str:
-        """Returns the artifact type."""
+        """Returns the artifact type.
+
+        Returns:
+            The artifact type.
+        """
         return self._type
 
     @property
     def data_type(self) -> str:
-        """Returns the data type of the artifact."""
+        """Returns the data type of the artifact.
+
+        Returns:
+            The data type of the artifact.
+        """
         return self._data_type
 
     @property
     def uri(self) -> str:
-        """Returns the URI where the artifact data is stored."""
+        """Returns the URI where the artifact data is stored.
+
+        Returns:
+            The URI where the artifact data is stored.
+        """
         return self._uri
 
     @property
     def parent_step_id(self) -> int:
-        """Returns the ID of the parent step. This need not be equivalent to
-        the ID of the producer step."""
+        """Returns the ID of the parent step.
+
+        This need not be equivalent to the ID of the producer step.
+
+        Returns:
+            The ID of the parent step.
+        """
         return self._parent_step_id
 
     @property
     def producer_step(self) -> "StepView":
-        """Returns the original StepView that produced the artifact."""
+        """Returns the original StepView that produced the artifact.
+
+        Returns:
+            The original StepView that produced the artifact.
+        """
         # TODO [ENG-174]: Replace with artifact.id instead of passing self if
         #  required.
         return self._metadata_store.get_producer_step_from_artifact(self)
 
     @property
     def is_cached(self) -> bool:
-        """Returns True if artifact was cached in a previous run, else False."""
+        """Returns True if artifact was cached in a previous run, else False.
+
+        Returns:
+            True if artifact was cached in a previous run, else False.
+        """
         # self._metadata_store.
         return self.producer_step.id != self.parent_step_id
 
@@ -121,17 +153,37 @@ class ArtifactView:
                 artifact during execution of the pipeline.
 
         Returns:
-              The materialized data.
+            The materialized data.
+
+        Raises:
+            ModuleNotFoundError: If the materializer class could not be found.
         """
         if not materializer_class:
-            materializer_class = source_utils.load_source_path_class(
-                self._materializer
-            )
+            try:
+                materializer_class = source_utils.load_source_path_class(
+                    self._materializer
+                )
+            except (ModuleNotFoundError, AttributeError) as e:
+                logger.error(
+                    f"ZenML can not locate and import the materializer module "
+                    f"{self._materializer} which was used to write this "
+                    f"artifact. If you want to read from it, please provide "
+                    f"a 'materializer_class'."
+                )
+                raise ModuleNotFoundError(e) from e
 
         if not output_data_type:
-            output_data_type = source_utils.load_source_path_class(
-                self._data_type
-            )
+            try:
+                output_data_type = source_utils.load_source_path_class(
+                    self._data_type
+                )
+            except (ModuleNotFoundError, AttributeError) as e:
+                logger.error(
+                    f"ZenML can not locate and import the data type of this "
+                    f"artifact {self._data_type}. If you want to read "
+                    f"from it, please provide a 'output_data_type'."
+                )
+                raise ModuleNotFoundError(e) from e
 
         logger.debug(
             "Using '%s' to read '%s' (uri: %s).",
@@ -147,7 +199,11 @@ class ArtifactView:
         return materializer.handle_input(output_data_type)
 
     def __repr__(self) -> str:
-        """Returns a string representation of this artifact."""
+        """Returns a string representation of this artifact.
+
+        Returns:
+            A string representation of this artifact.
+        """
         return (
             f"{self.__class__.__qualname__}(id={self._id}, "
             f"type='{self._type}', uri='{self._uri}', "
@@ -155,8 +211,15 @@ class ArtifactView:
         )
 
     def __eq__(self, other: Any) -> bool:
-        """Returns whether the other object is referring to the
-        same artifact."""
+        """Returns whether the other object is referring to the same artifact.
+
+        Args:
+            other: The other object to compare to.
+
+        Returns:
+            True if the other object is referring to the same artifact, else
+            False.
+        """
         if isinstance(other, ArtifactView):
             return self._id == other._id and self._uri == other._uri
         return NotImplemented
