@@ -1,10 +1,28 @@
+#  Copyright (c) ZenML GmbH 2022. All Rights Reserved.
+#
+#  Licensed under the Apache License, Version 2.0 (the "License");
+#  you may not use this file except in compliance with the License.
+#  You may obtain a copy of the License at:
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
+#  or implied. See the License for the specific language governing
+#  permissions and limitations under the License.
+"""Step context class."""
+
 from typing import TYPE_CHECKING, Dict, NamedTuple, Optional, Type, cast
 
 from zenml.exceptions import StepContextError
+from zenml.repository import Repository
 
 if TYPE_CHECKING:
     from zenml.artifacts.base_artifact import BaseArtifact
     from zenml.materializers.base_materializer import BaseMaterializer
+    from zenml.metadata_stores.base_metadata_store import BaseMetadataStore
+    from zenml.stack import Stack
 
 
 class StepContextOutput(NamedTuple):
@@ -17,18 +35,26 @@ class StepContextOutput(NamedTuple):
 class StepContext:
     """Provides additional context inside a step function.
 
-    This class is used to access materializers and artifact URIs inside
-    a step function. To use it, add a `StepContext` object to the signature
-    of your step function like this:
+    This class is used to access the metadata store, materializers and
+    artifacts inside a step function. To use it, add a `StepContext` object
+    to the signature of your step function like this:
 
+    ```python
     @step
     def my_step(context: StepContext, ...)
         context.get_output_materializer(...)
+    ```
 
     You do not need to create a `StepContext` object yourself and pass it
     when creating the step, as long as you specify it in the signature ZenML
     will create the `StepContext` and automatically pass it when executing your
     step.
+
+    **Note**: When using a `StepContext` inside a step, ZenML disables caching
+    for this step by default as the context provides access to external
+    resources which might influence the result of your step execution. To
+    enable caching anyway, explicitly enable it in the `@step` decorator or when
+    initializing your custom step class.
     """
 
     def __init__(
@@ -42,13 +68,13 @@ class StepContext:
         Args:
             step_name: The name of the step that this context is used in.
             output_materializers: The output materializers of the step that
-                this context is used in.
+                                  this context is used in.
             output_artifacts: The output artifacts of the step that this
-                context is used in.
+                              context is used in.
 
         Raises:
-             StepInterfaceError: If the keys of the output materializers and
-             output artifacts do not match.
+             StepContextError: If the keys of the output materializers and
+                               output artifacts do not match.
         """
         if output_materializers.keys() != output_artifacts.keys():
             raise StepContextError(
@@ -65,6 +91,8 @@ class StepContext:
             )
             for key in output_materializers.keys()
         }
+        self._metadata_store = Repository().active_stack.metadata_store
+        self._stack = Repository().active_stack
 
     def _get_output(
         self, output_name: Optional[str] = None
@@ -80,9 +108,9 @@ class StepContext:
             given output.
 
         Raises:
-            StepInterfaceError: If the step has no outputs, no output for
-                the given `output_name` or if no `output_name` was given but
-                the step has multiple outputs.
+            StepContextError: If the step has no outputs, no output for
+                              the given `output_name` or if no `output_name`
+                              was given but the step has multiple outputs.
         """
         output_count = len(self._outputs)
         if output_count == 0:
@@ -110,6 +138,28 @@ class StepContext:
         else:
             return next(iter(self._outputs.values()))
 
+    @property
+    def metadata_store(self) -> "BaseMetadataStore":
+        """Returns the metadata store.
+
+        Returns an instance of the metadata store that is used to store
+        metadata about the step (and the corresponding pipeline) which is
+        being executed.
+
+        Returns:
+            The metadata store.
+        """
+        return self._metadata_store
+
+    @property
+    def stack(self) -> Optional["Stack"]:
+        """Returns the current active stack.
+
+        Returns:
+            The current active stack or None.
+        """
+        return self._stack
+
     def get_output_materializer(
         self,
         output_name: Optional[str] = None,
@@ -130,11 +180,6 @@ class StepContext:
         Returns:
             A materializer initialized with the output artifact for
             the given output.
-
-        Raises:
-            StepInterfaceError: If the step has no outputs, no output for
-                the given `output_name` or if no `output_name` was given but
-                the step has multiple outputs.
         """
         materializer_class, artifact = self._get_output(output_name)
         # use custom materializer class if provided or fallback to default
@@ -153,10 +198,5 @@ class StepContext:
 
         Returns:
             Artifact URI for the given output.
-
-        Raises:
-            StepInterfaceError: If the step has no outputs, no output for
-                the given `output_name` or if no `output_name` was given but
-                the step has multiple outputs.
         """
         return cast(str, self._get_output(output_name).artifact.uri)
