@@ -4,7 +4,7 @@
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at:
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#       https://www.apache.org/licenses/LICENSE-2.0
 #
 #  Unless required by applicable law or agreed to in writing, software
 #  distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,7 +12,8 @@
 #  or implied. See the License for the specific language governing
 #  permissions and limitations under the License.
 
-"""
+"""Utility functions for Steps.
+
 The collection of utility functions/classes are inspired by their original
 implementation of the Tensorflow Extended team, which can be found here:
 
@@ -28,7 +29,9 @@ from __future__ import absolute_import, division, print_function
 import inspect
 import json
 import sys
+import typing
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     ClassVar,
@@ -38,36 +41,49 @@ from typing import (
     KeysView,
     List,
     Optional,
+    Sequence,
     Set,
     Type,
     ValuesView,
+    cast,
 )
 
 import pydantic
 from tfx.dsl.component.experimental.decorators import _SimpleComponent
 from tfx.dsl.components.base.base_executor import BaseExecutor
 from tfx.dsl.components.base.executor_spec import ExecutorClassSpec
+from tfx.orchestration.portable import outputs_utils
+from tfx.proto.orchestration import execution_result_pb2
 from tfx.types import component_spec
 from tfx.types.channel import Channel
 from tfx.utils import json_utils
 
 from zenml.artifacts.base_artifact import BaseArtifact
-from zenml.exceptions import MissingStepParameterError
+from zenml.exceptions import MissingStepParameterError, StepInterfaceError
+from zenml.io import fileio
 from zenml.logger import get_logger
 from zenml.materializers.base_materializer import BaseMaterializer
 from zenml.steps.base_step_config import BaseStepConfig
 from zenml.steps.step_context import StepContext
+from zenml.steps.step_environment import StepEnvironment
 from zenml.steps.step_output import Output
 from zenml.utils import source_utils
 
+if TYPE_CHECKING:
+    from zenml.steps.base_step import BaseStep
+
 logger = get_logger(__name__)
 
-STEP_INNER_FUNC_NAME: str = "process"
+STEP_INNER_FUNC_NAME: str = "entrypoint"
 SINGLE_RETURN_OUT_NAME: str = "output"
 PARAM_STEP_NAME: str = "step_name"
 PARAM_ENABLE_CACHE: str = "enable_cache"
 PARAM_PIPELINE_PARAMETER_NAME: str = "pipeline_parameter_name"
+PARAM_CREATED_BY_FUNCTIONAL_API: str = "created_by_functional_api"
+PARAM_CUSTOM_STEP_OPERATOR: str = "custom_step_operator"
 INTERNAL_EXECUTION_PARAMETER_PREFIX: str = "zenml-"
+INSTANCE_CONFIGURATION: str = "INSTANCE_CONFIGURATION"
+OUTPUT_SPEC: str = "OUTPUT_SPEC"
 
 
 def do_types_match(type_a: Type[Any], type_b: Type[Any]) -> bool:
@@ -85,6 +101,32 @@ def do_types_match(type_a: Type[Any], type_b: Type[Any]) -> bool:
     return type_a == type_b
 
 
+def resolve_type_annotation(obj: Any) -> Any:
+    """Returns the non-generic class for generic aliases of the typing module.
+
+    If the input is no generic typing alias, the input itself is returned.
+
+    Example: if the input object is `typing.Dict`, this method will return the
+    concrete class `dict`.
+
+    Args:
+        obj: The object to resolve.
+
+    Returns:
+        The non-generic class for generic aliases of the typing module.
+    """
+    from typing import _GenericAlias  # type: ignore[attr-defined]
+
+    if sys.version_info >= (3, 8):
+        return typing.get_origin(obj) or obj
+    else:
+        # python 3.7
+        if isinstance(obj, _GenericAlias):
+            return obj.__origin__
+        else:
+            return obj
+
+
 def generate_component_spec_class(
     step_name: str,
     input_spec: Dict[str, Type[BaseArtifact]],
@@ -185,6 +227,7 @@ def generate_component_class(
 
 class _PropertyDictWrapper(json_utils.Jsonable):
     """Helper class to wrap inputs/outputs from TFX nodes.
+
     Currently, this class is read-only (setting properties is not implemented).
     Internal class: no backwards compatibility guarantees.
     Code Credit: https://github.com/tensorflow/tfx/blob
@@ -200,23 +243,44 @@ class _PropertyDictWrapper(json_utils.Jsonable):
 
         Args:
             data: The data to be wrapped.
-            compat_aliases: Compatability aliases to support deprecated keys.
+            compat_aliases: Compatibility aliases to support deprecated keys.
         """
         self._data = data
         self._compat_aliases = compat_aliases or {}
 
     def __iter__(self) -> Iterator[str]:
-        """Returns a generator that yields keys of the wrapped dictionary."""
+        """Returns a generator that yields keys of the wrapped dictionary.
+
+        Yields:
+            Keys of the wrapped dictionary.
+        """
         yield from self._data
 
     def __getitem__(self, key: str) -> Channel:
-        """Returns the dictionary value for the specified key."""
+        """Returns the dictionary value for the specified key.
+
+        Args:
+            key: The key to look up.
+
+        Returns:
+            The dictionary value for the specified key.
+        """
         if key in self._compat_aliases:
             key = self._compat_aliases[key]
         return self._data[key]
 
     def __getattr__(self, key: str) -> Channel:
-        """Returns the dictionary value for the specified key."""
+        """Returns the dictionary value for the specified key.
+
+        Args:
+            key: The key to look up.
+
+        Returns:
+            The dictionary value for the specified key.
+
+        Raises:
+            AttributeError: If the key is not found.
+        """
         if key in self._compat_aliases:
             key = self._compat_aliases[key]
         try:
@@ -225,23 +289,43 @@ class _PropertyDictWrapper(json_utils.Jsonable):
             raise AttributeError
 
     def __repr__(self) -> str:
-        """Returns the representation of the wrapped dictionary."""
+        """Returns the representation of the wrapped dictionary.
+
+        Returns:
+            The representation of the wrapped dictionary.
+        """
         return repr(self._data)
 
     def get_all(self) -> Dict[str, Channel]:
-        """Returns the wrapped dictionary."""
+        """Returns the wrapped dictionary.
+
+        Returns:
+            The wrapped dictionary.
+        """
         return self._data
 
     def keys(self) -> KeysView[str]:
-        """Returns the keys of the wrapped dictionary."""
+        """Returns the keys of the wrapped dictionary.
+
+        Returns:
+            The keys of the wrapped dictionary.
+        """
         return self._data.keys()
 
     def values(self) -> ValuesView[Channel]:
-        """Returns the values of the wrapped dictionary."""
+        """Returns the values of the wrapped dictionary.
+
+        Returns:
+            The values of the wrapped dictionary.
+        """
         return self._data.values()
 
     def items(self) -> ItemsView[str, Channel]:
-        """Returns the items of the wrapped dictionary."""
+        """Returns the items of the wrapped dictionary.
+
+        Returns:
+            The items of the wrapped dictionary.
+        """
         return self._data.items()
 
 
@@ -250,12 +334,16 @@ class _ZenMLSimpleComponent(_SimpleComponent):
 
     @property
     def outputs(self) -> _PropertyDictWrapper:  # type: ignore[override]
-        """Returns the wrapped spec outputs."""
+        """Returns the wrapped spec outputs.
+
+        Returns:
+            The wrapped spec outputs.
+        """
         return _PropertyDictWrapper(self.spec.outputs)
 
 
 class _FunctionExecutor(BaseExecutor):
-    """Base TFX Executor class which is compatible with ZenML steps"""
+    """Base TFX Executor class which is compatible with ZenML steps."""
 
     _FUNCTION = staticmethod(lambda: None)
     materializers: ClassVar[
@@ -274,9 +362,12 @@ class _FunctionExecutor(BaseExecutor):
         Returns:
             The right materializer based on the defaults or optionally the one
             set by the user.
+
+        Raises:
+            ValueError: If the materializer is not found.
         """
         if not self.materializers:
-            raise ValueError("Materializers are missing is not set!")
+            raise ValueError("Materializers are not set!")
 
         materializer_class = self.materializers[param_name]
         return materializer_class
@@ -284,8 +375,9 @@ class _FunctionExecutor(BaseExecutor):
     def resolve_input_artifact(
         self, artifact: BaseArtifact, data_type: Type[Any]
     ) -> Any:
-        """Resolves an input artifact, i.e., reading it from the Artifact Store
-        to a pythonic object.
+        """Resolves an input artifact.
+
+        This method reads it from the Artifact Store to a Pythonic object.
 
         Args:
             artifact: A TFX artifact type.
@@ -294,6 +386,17 @@ class _FunctionExecutor(BaseExecutor):
         Returns:
             Return the output of `handle_input()` of selected materializer.
         """
+        # Skip materialization for BaseArtifact and its subtypes.
+        if issubclass(data_type, BaseArtifact):
+            if data_type != type(artifact):
+                logger.warning(
+                    f"You specified the data_type `{data_type}` but the actual "
+                    f"artifact type from the previous step is "
+                    f"`{type(artifact)}`. Ignoring this for now, but please be "
+                    f"aware of this in your step code."
+                )
+            return artifact
+
         materializer = source_utils.load_source_path_class(
             artifact.materializer
         )(artifact)
@@ -303,14 +406,20 @@ class _FunctionExecutor(BaseExecutor):
     def resolve_output_artifact(
         self, param_name: str, artifact: BaseArtifact, data: Any
     ) -> None:
-        """Resolves an output artifact, i.e., writing it to the Artifact Store.
-        Calls `handle_return(return_values)` of the selected materializer.
+        """Resolves an output artifact.
+
+        This writes it to the Artifact Store. Calls
+        `handle_return(return_values)` of the selected materializer.
 
         Args:
             param_name: Name of output param.
             artifact: A TFX artifact type.
             data: The object to be passed to `handle_return()`.
         """
+        # Skip materialization for BaseArtifact and subclasses.
+        if issubclass(type(data), BaseArtifact):
+            return
+
         materializer_class = self.resolve_materializer_with_registry(
             param_name, artifact
         )
@@ -326,10 +435,10 @@ class _FunctionExecutor(BaseExecutor):
         Args:
             output_value: Value of output.
             specified_type: What the type of output should be as defined in the
-            signature.
+                signature.
 
         Raises:
-            ValueError if types dont match.
+            ValueError: if types do not match.
         """
         # TODO [ENG-160]: Include this check when we figure out the logic of
         #  slightly different subclasses.
@@ -346,13 +455,20 @@ class _FunctionExecutor(BaseExecutor):
         output_dict: Dict[str, List[BaseArtifact]],
         exec_properties: Dict[str, Any],
     ) -> None:
-        """Main block for the execution of the step
+        """Main block for the execution of the step.
 
         Args:
             input_dict: dictionary containing the input artifacts
             output_dict: dictionary containing the output artifacts
             exec_properties: dictionary containing the execution parameters
+
+        Raises:
+            MissingStepParameterError: if a required parameter is missing.
+            RuntimeError: if the step fails.
+            StepInterfaceError: if the step interface is not implemented.
         """
+        step_name = getattr(self, PARAM_STEP_NAME)
+
         # remove all ZenML internal execution properties
         exec_properties = {
             k: json.loads(v)
@@ -360,32 +476,40 @@ class _FunctionExecutor(BaseExecutor):
             if not k.startswith(INTERNAL_EXECUTION_PARAMETER_PREFIX)
         }
 
-        # Building the args for the process function
+        # Building the args for the entrypoint function
         function_params = {}
 
         # First, we parse the inputs, i.e., params and input artifacts.
-        spec = inspect.getfullargspec(self._FUNCTION)
+        spec = inspect.getfullargspec(inspect.unwrap(self._FUNCTION))
         args = spec.args
+
+        if args and args[0] == "self":
+            args.pop(0)
+
         for arg in args:
             arg_type = spec.annotations.get(arg, None)
+            arg_type = resolve_type_annotation(arg_type)
+
             if issubclass(arg_type, BaseStepConfig):
                 try:
                     config_object = arg_type.parse_obj(exec_properties)
                 except pydantic.ValidationError as e:
                     missing_fields = [
-                        field
+                        str(field)
                         for error_dict in e.errors()
                         for field in error_dict["loc"]
                     ]
 
                     raise MissingStepParameterError(
-                        getattr(self, PARAM_STEP_NAME), missing_fields, arg_type
+                        step_name,
+                        missing_fields,
+                        arg_type,
                     ) from None
                 function_params[arg] = config_object
             elif issubclass(arg_type, StepContext):
                 output_artifacts = {k: v[0] for k, v in output_dict.items()}
-                context = StepContext(
-                    step_name=getattr(self, PARAM_STEP_NAME),
+                context = arg_type(
+                    step_name=step_name,
                     output_materializers=self.materializers or {},
                     output_artifacts=output_artifacts,
                 )
@@ -396,22 +520,146 @@ class _FunctionExecutor(BaseExecutor):
                     input_dict[arg][0], arg_type
                 )
 
-        return_values = self._FUNCTION(**function_params)
-        spec = inspect.getfullargspec(self._FUNCTION)
+        if self._context is None:
+            raise RuntimeError(
+                "No TFX context is set for the currently running pipeline. "
+                "Cannot retrieve pipeline runtime information."
+            )
+        # Wrap the execution of the step function in a step environment
+        # that the step function code can access to retrieve information about
+        # the pipeline runtime, such as the current step name and the current
+        # pipeline run ID
+        with StepEnvironment(
+            pipeline_name=self._context.pipeline_info.id,
+            pipeline_run_id=self._context.pipeline_run_id,
+            step_name=getattr(self, PARAM_STEP_NAME),
+        ):
+            return_values = self._FUNCTION(**function_params)
+
+        spec = inspect.getfullargspec(inspect.unwrap(self._FUNCTION))
         return_type: Type[Any] = spec.annotations.get("return", None)
         if return_type is not None:
             if isinstance(return_type, Output):
-                # Resolve named (and multi-) outputs.
-                for i, output_tuple in enumerate(return_type.items()):
-                    self.resolve_output_artifact(
-                        output_tuple[0],
-                        output_dict[output_tuple[0]][0],
-                        return_values[i],  # order preserved.
-                    )
+                output_annotations = list(return_type.items())
             else:
-                # Resolve single output
-                self.resolve_output_artifact(
-                    SINGLE_RETURN_OUT_NAME,
-                    output_dict[SINGLE_RETURN_OUT_NAME][0],
-                    return_values,
+                output_annotations = [(SINGLE_RETURN_OUT_NAME, return_type)]
+
+            # if there is only one output annotation (either directly specified
+            # or contained in an `Output` tuple) we treat the step function
+            # return value as the return for that output
+            if len(output_annotations) == 1:
+                return_values = [return_values]
+            elif not isinstance(return_values, Sequence):
+                # if the user defined multiple outputs, the return value must
+                # be a sequence
+                raise StepInterfaceError(
+                    f"Wrong step function output type for step '{step_name}: "
+                    f"Expected multiple outputs ({output_annotations}) but "
+                    f"the function did not return a sequence-like object "
+                    f"(actual return value: {return_values})."
                 )
+            elif len(output_annotations) != len(return_values):
+                # if the user defined multiple outputs, the amount of actual
+                # outputs must be the same
+                raise StepInterfaceError(
+                    f"Wrong amount of step function outputs for step "
+                    f"'{step_name}: Expected {len(output_annotations)} outputs "
+                    f"but the function returned {len(return_values)} outputs"
+                    f"(return values: {return_values})."
+                )
+
+            for return_value, (output_name, output_type) in zip(
+                return_values, output_annotations
+            ):
+                if not isinstance(return_value, output_type):
+                    raise StepInterfaceError(
+                        f"Wrong type for output '{output_name}' of step "
+                        f"'{step_name}' (expected type: {output_type}, "
+                        f"actual type: {type(return_value)})."
+                    )
+
+                self.resolve_output_artifact(
+                    output_name, output_dict[output_name][0], return_value
+                )
+
+        # Write the executor output to the artifact store so the executor
+        # operator (potentially not running on the same machine) can read it
+        # to populate the metadata store
+        executor_output = execution_result_pb2.ExecutorOutput()
+        outputs_utils.populate_output_artifact(executor_output, output_dict)
+
+        logger.debug(
+            "Writing executor output to '%s'.",
+            self._context.executor_output_uri,
+        )
+        with fileio.open(self._context.executor_output_uri, "wb") as f:
+            f.write(executor_output.SerializeToString())
+
+
+def clone_step(step: Type["BaseStep"], step_name: str) -> Type["BaseStep"]:
+    """Returns a clone of the given step.
+
+    Call this function when you want to use several copies of the same step
+    in a pipeline.
+
+    Args:
+        step: the step to clone
+        step_name: name to use for the cloned step. This name will be used
+            to uniquely identify the cloned step in the Python module from
+            which this function is called.
+
+    Returns:
+        A clone of the given step.
+
+    Raises:
+        TypeError: if the given step is not a subclass of BaseStep.
+        ImportError: if the the calling Python module cannot be determined
+            or if the given step name cannot be used because the calling
+            Python module already contains a symbol with the same name.
+    """
+    from zenml.steps.base_step import BaseStep
+
+    if not issubclass(step, BaseStep):
+        raise TypeError(
+            f"The step source `{step}` is not a `{BaseStep}` subclass."
+        )
+
+    # get the python module from which this function is called
+    frm = inspect.stack()[1]
+    mod = inspect.getmodule(frm[0])
+
+    if not mod:
+        raise ImportError(
+            f"Cannot clone step `{step}` because the calling Python module "
+            f"cannot be determined."
+        )
+
+    if step_name in mod.__dict__:
+        raise ImportError(
+            f"The step name `{step_name}` cannot be used because the "
+            f"`{mod.__name__}` Python module already contains an identifier "
+            f"with the same name. Try using a different step name."
+        )
+
+    config = getattr(step, INSTANCE_CONFIGURATION, {}).copy()
+    # mark the step as being created by the functional API,
+    # otherwise the base class will try to use the class source code
+    # to compute the hash for the cache key (instead of the entrypoint
+    # source code)
+    config[PARAM_CREATED_BY_FUNCTIONAL_API] = True
+
+    step_clone = cast(
+        Type[BaseStep],
+        type(  # noqa
+            step_name,
+            (step,),
+            {
+                INSTANCE_CONFIGURATION: config,
+                "__module__": mod.__name__,
+                "__doc__": step.__doc__,
+            },
+        ),
+    )
+
+    mod.__dict__[step_name] = step_clone
+    return step_clone
